#######################################################################################################################################
#R code for analysis supporting the comment of Bertrand et al. (2020) on the work of Zellweger et al. (2020)
#Every results are provided here: https://figshare.com/s/07a39af48116a3126689 ; in order to directly analyse the results without launching computing demanding analysis every table saved below are provided.

#The first part of the script is R codes for the analysis of the all species dataset (supporting results in the Technical comments).
#The second part of the script is R codes for the analysis of the common species dataset (results are not discussed in the Technical comment but we observed the same pattern as observed when all species are considered).

#contact: romain.bertrand2@univ-tlse3.fr
#######################################################################################################################################

###Requiered R packages for analysis###
library(MuMIn) #version 1.43.17
library(lme4) #version 1.1-23
library(optimx) #version 2040-4.2
library(variancePartition) #version 1.16.1
library(data.table) #version 1.12.8

#########################analysis of all species##############################
###input repository definition###
inF="..." #please set the correct reprository address where data of Zellweger et al. (2020) are located on your computer

###data and function loading###
#data of Zellweger et al. 2020 are available here: https://doi.org/10.5061/dryad.r7sqv9s83
setwd(inF)
t1=read.csv2("DataThermophilisation_allSpecies_Zellweger_et_al_Microclimate.csv",sep=",",dec=".",h=T) #all species
source("Bertrand_et_al_function.R") #please put this R code file at inF address

###variable computation###
t1$microD=t1$MicroTmaxChangerate-t1$Thermophi_95pctl #the microclimatic debt as computed by Zellweger et al. (2020)
t1$Tbuffd=(t1$TmaxBufResurvey-t1$TmaxBufBaseline) * -1 #the change in temperature buffering by canopy closure and openness as computed by Zellweger et al. (2020)

###Thermophilization analysis###
#testing for the effect of changes in microclimate conditions on the thermophilization of forest plant communities (95th percentiles)
## model of Zellenberg et al. (2020) supporting their Fig. 2 (lines 169-173 of Rcode_Zellweger_et_al.R)
M2<-lmer(Thermophi_95pctl ~ scale(MicroTmaxChangerate) + (1|region),data=t1,na.action=na.omit,REML=T)
r.squaredGLMM(M2) #R2m=0.01070619 & R2c=0.1177441

##Analsys of Bertrand et al. 2020
###output repository definition###
outF="..." #please set the correct reprository address where you want to save results (give a different address for each analysis)

#testing for the effects of macroclimate change and changes in temperature buffering by canopy closure and openness on the thermophilization observed in forest plant communities (95th percentiles)
###1#is colinearity between macroclimate change and temperature buffering change?
m1=lmer(scale(Tbuffd)~scale(MacroTmaxChangerate)+(1|region),data=t1,REML=T,na.action="na.fail",control = lmerControl(optimizer ='optimx', optCtrl=list(method='nlminb')))
r.squaredGLMM(m1) #R2m=0.006496959 & R2c=0.2466336
###No collinearity

###1#LMM
mX=lmer(Thermophi_95pctl~scale(MacroTmaxChangerate)*scale(Tbuffd)+(1|region),data=t1,REML=T,na.action="na.fail",control = lmerControl(optimizer ='optimx', optCtrl=list(method='nlminb')))
summary(mX)
r.squaredGLMM(mX) #R2m=0.006097199 & R2c=0.1031857

###2#Computing bootstraps to infer 95% confidence intervals of R2 (R2c, R2M and part of variance explained by each direct effects using the function calcVarPart from the variancePartition package) and size effects, and also to test for significance of each direct effects.
r2=NULL
nB=12000 #set the bootstrap number
s1=5 #the seed
ncpu=10 #the number of cpu used for parallel computation

#WARNING: it can take a long time to run. It takes ~10min on 10 CPUs of a Intel® Xeon(R) E-2286M CPU @ 2.40GHz × 16 computer with 32Go memory (ubuntu 18.04)
t0=Sys.time()
boot1.time <- system.time(
  boot1 <- bootMer2(mX, statF, nsim=nB, use.u=F, type="parametric",ncpus=ncpu,seed=s1,parallel="multicore",.progress = "none")
)
print(Sys.time()-t0)

#exporting results
ess=simplify2array(boot1)
r2=rbindlist(ess[1,])
coeff=rbindlist(ess[2,])
rand=rbindlist(ess[3,])
rand2=rbindlist(ess[4,])
vPart=rbindlist(ess[5,])

setwd(outF)  
write.table(r2,"r2.csv",sep=";",dec=".",row=F)
#output:
#r2m= marginal R2 of every bootstrap model (each line is a different model)
#r2c= conditional R2 of every bootstrap model
#warnM= inform for warning during the model fit (such as convergence issue)
#sing= output of the model singularity test (FALSE= no signularity; TRUE= singularity issue)

write.table(coeff,"coeff.csv",sep=";",dec=".",row=F)
#output is the result of model$coefficient for everey bootstrap model:
#Estimate= coefficient value computed from every bootstrap model
#Std..Error= Standard error computed from every bootstrap model
#t.value= t-value computed from every bootstrap model
#var= names of the term

write.table(rand2,"randVar.csv",sep=";",dec=".",row=F)
#output is the result of summary(.)$varcor for every bootstrap model:
#var= names of the term
#vcov= Variance of random intercept

write.table(vPart,"vPart.csv",sep=";",dec=".",row=F)
#output is the result of variance partitioning inferred from calcVarPart function (variancePartition package)
#each columns is a variable
#each row is a bootstrap iteration

###3# Bootstrap analysis
setwd(outF) 
vPart=read.csv2("vPart.csv",sep=";",dec=".",h=T)
coeff=read.csv2("coeff.csv",sep=";",dec=".",h=T)
rand2=read.csv2("randVar.csv",sep=";",dec=".",h=T)
r2=read.csv2("r2.csv",sep=";",dec=".",h=T)

#bootstrap test: H0= coefficient equal to 0; H1= coefficient higher than 0
test1=function(x,mu=0){
  return(1-(length(x[x>mu])/length(x)))
}

#bootstrap test: H0= coefficient equal to 0; H1= coefficient lower than 0
test2=function(x,mu=0){
  return(1-(length(x[x<mu])/length(x)))
}

nB2=10000 #the number of bootstrap selection.
#Some of the 12000 boostraps previously conducted can be biased by singularity or model convergence issues.
#So we randomly selected the 10000 bootstraps withouth any fitting issues
s1=10 #the random seed
r2$nB=1:nrow(r2)
r2a=subset(r2,is.na(warnM)==T)
if(nrow(r2a)>=nB2){
  set.seed(s1)
  r2a=r2a[sample(1:nrow(r2a),size=nB2,replace=F),]
}
if(nrow(r2a)<nB2){
  print(paste("sampling size is less than ",nB2," bootstraps",sep=""))
}else{
  write.table(r2a,paste("r2_",nB2,"boot.csv",sep=""),sep=";",dec=".",row=F) #the list of plots selected for each boostrap iteration
  resR2=data.frame(type="r2m",moy=mean(r2a$r2m),sd=sd(r2a$r2m),median=median(r2a$r2m),q025=quantile(r2a$r2m,probs=0.025),q975=quantile(r2a$r2m,probs=0.975))
  resR2=rbind(resR2,data.frame(type="r2c",moy=mean(r2a$r2c),sd=sd(r2a$r2c),median=median(r2a$r2c),q025=quantile(r2a$r2c,probs=0.025),q975=quantile(r2a$r2c,probs=0.975)))
  resR2$n_sing=nrow(subset(r2a,sing==T))
  
  c1=coeff
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res=data.frame(var=names(tapply(rr3$Estimate,rr3$var,mean)),moy=tapply(rr3$Estimate,rr3$var,mean),sd=tapply(rr3$Estimate,rr3$var,sd),median=tapply(rr3$Estimate,rr3$var,median),q025=tapply(rr3$Estimate,rr3$var,quantile,probs=0.025),
                 p975=tapply(rr3$Estimate,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$Estimate,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$Estimate,rr3$var,test1,mu=0))
  
  c1=rand2
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res2=data.frame(var=names(tapply(rr3$vcov,rr3$var,mean)),moy=tapply(rr3$vcov,rr3$var,mean),sd=tapply(rr3$vcov,rr3$var,sd),median=tapply(rr3$vcov,rr3$var,median),q025=tapply(rr3$vcov,rr3$var,quantile,probs=0.025),
                  p975=tapply(rr3$vcov,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$vcov,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$vcov,rr3$var,test1,mu=0))
  
  c1=vPart
  c1$nB=1:nrow(c1)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  resR2p=data.frame(var=names(apply(rr3[,2:ncol(rr3)],2,mean)),moy=apply(rr3[,2:ncol(rr3)],2,mean),sd=apply(rr3[,2:ncol(rr3)],2,sd),median=apply(rr3[,2:ncol(rr3)],2,median),q025=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.025),q975=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.975))
  
}

write.table(resR2,"summary_R2.csv",sep=";",dec=".",row=F)
#R2 statistic computed from 10000 boostrap model:
#type= R2 type (r2m= marginal R2; r2c= conditional R2)
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution
#n_sing= number of singular model


write.table(res,"summary_coeff.csv",sep=";",dec=".",row=F)
#coefficient statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap coefficient distribution
#q975 = 97.5% quantile of the bootstrap coefficient distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient lower than 0)


write.table(res2,"summary_randI.csv",sep=";",dec=".",row=F)
#random intercept statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap random intercept distribution
#q975 = 97.5% quantile of the bootstrap random intercept distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient lower than 0)

write.table(resR2p,"summary_R2p.csv",sep=";",dec=".",row=F)
#R2 partitioning computed from 10000 bootstrap model:
#var= Term of the model
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution

###Microclimatic debt analysis###
#Analsys of Bertrand et al. 2020
###output repository definition###
outF="..." #please set the correct reprository address where you want to save results (give a different address for each analysis)

#testing for the effects of macroclimate change and changes in temperature buffering by canopy closure and openness on the microclimatic debt observed in forest plant communities (95th percentiles)
###1#LMM
mX=lmer(microD~scale(MacroTmaxChangerate)*scale(Tbuffd)+(1|region),data=t1,REML=T,na.action="na.fail",control = lmerControl(optimizer ='optimx', optCtrl=list(method='nlminb')))
summary(mX)
r.squaredGLMM(mX) #R2m=0.7710363 & R2c=0.8116492

###2#Computing bootstraps to infer 95% confidence intervals of R2 (R2c, R2M and part of variance explained by each direct effects using the function calcVarPart from the variancePartition package) and size effects, and also to test for significance of each direct effects.
r2=NULL
nB=12000 #set the bootstrap number
s1=5 #the seed
ncpu=10 #the number of cpu used for parallel computation

#WARNING: it can take a long time to run. It takes ~13min on 10 CPUs of a Intel® Xeon(R) E-2286M CPU @ 2.40GHz × 16 computer with 32Go memory (ubuntu 18.04)
t0=Sys.time()
boot1.time <- system.time(
  boot1 <- bootMer2(mX, statF, nsim=nB, use.u=F, type="parametric",ncpus=ncpu,seed=s1,parallel="multicore",.progress = "none")
)
print(Sys.time()-t0)

#exporting results
ess=simplify2array(boot1)
r2=rbindlist(ess[1,])
coeff=rbindlist(ess[2,])
rand=rbindlist(ess[3,])
rand2=rbindlist(ess[4,])
vPart=rbindlist(ess[5,])

setwd(outF)  
write.table(r2,"r2.csv",sep=";",dec=".",row=F)
#output:
#r2m= marginal R2 of every bootstrap model (each line is a different model)
#r2c= conditional R2 of every bootstrap model
#warnM= inform for warning during the model fit (such as convergence issue)
#sing= output of the model singularity test (FALSE= no signularity; TRUE= singularity issue)

write.table(coeff,"coeff.csv",sep=";",dec=".",row=F)
#output is the result of model$coefficient for everey bootstrap model:
#Estimate= coefficient value computed from every bootstrap model
#Std..Error= Standard error computed from every bootstrap model
#t.value= t-value computed from every bootstrap model
#var= names of the term

write.table(rand2,"randVar.csv",sep=";",dec=".",row=F)
#output is the result of summary(.)$varcor for everey bootstrap model:
#var= names of the term
#vcov= Variance of random intercept

write.table(vPart,"vPart.csv",sep=";",dec=".",row=F)
#output is the result of variance partitioning inferred from calcVarPart function (variancePartition package)
#each columns is a variable
#each row is a boostrap iteration

###3# Bootstrap analysis
setwd(outF) 
vPart=read.csv2("vPart.csv",sep=";",dec=".",h=T)
coeff=read.csv2("coeff.csv",sep=";",dec=".",h=T)
rand2=read.csv2("randVar.csv",sep=";",dec=".",h=T)
r2=read.csv2("r2.csv",sep=";",dec=".",h=T)

#bootstrap test: H0= coefficient equal to 0; H1= coefficient higher than 0
test1=function(x,mu=0){
  return(1-(length(x[x>mu])/length(x)))
}

#bootstrap test: H0= coefficient equal to 0; H1= coefficient lower than 0
test2=function(x,mu=0){
  return(1-(length(x[x<mu])/length(x)))
}

nB2=10000 #the number of bootstrap selection.
#Some of the 12000 bootstraps previously conducted can be biased by singularity or model convergence issues.
#So we selected the 10000 bootstraps withouth any fitting issues
s1=10 #the random seed
r2$nB=1:nrow(r2)
r2a=subset(r2,is.na(warnM)==T)
if(nrow(r2a)>=nB2){
  set.seed(s1)
  r2a=r2a[sample(1:nrow(r2a),size=nB2,replace=F),]
}
if(nrow(r2a)<nB2){
  print(paste("sampling size is less than ",nB2," bootstraps",sep=""))
}else{
  write.table(r2a,paste("r2_",nB2,"boot.csv",sep=""),sep=";",dec=".",row=F) #the list of plots selected for each boostrap iteration
  resR2=data.frame(type="r2m",moy=mean(r2a$r2m),sd=sd(r2a$r2m),median=median(r2a$r2m),q025=quantile(r2a$r2m,probs=0.025),q975=quantile(r2a$r2m,probs=0.975))
  resR2=rbind(resR2,data.frame(type="r2c",moy=mean(r2a$r2c),sd=sd(r2a$r2c),median=median(r2a$r2c),q025=quantile(r2a$r2c,probs=0.025),q975=quantile(r2a$r2c,probs=0.975)))
  resR2$n_sing=nrow(subset(r2a,sing==T))
  
  c1=coeff
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res=data.frame(var=names(tapply(rr3$Estimate,rr3$var,mean)),moy=tapply(rr3$Estimate,rr3$var,mean),sd=tapply(rr3$Estimate,rr3$var,sd),median=tapply(rr3$Estimate,rr3$var,median),q025=tapply(rr3$Estimate,rr3$var,quantile,probs=0.025),
                 p975=tapply(rr3$Estimate,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$Estimate,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$Estimate,rr3$var,test1,mu=0))
  
  c1=rand2
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res2=data.frame(var=names(tapply(rr3$vcov,rr3$var,mean)),moy=tapply(rr3$vcov,rr3$var,mean),sd=tapply(rr3$vcov,rr3$var,sd),median=tapply(rr3$vcov,rr3$var,median),q025=tapply(rr3$vcov,rr3$var,quantile,probs=0.025),
                  p975=tapply(rr3$vcov,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$vcov,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$vcov,rr3$var,test1,mu=0))
  
  c1=vPart
  c1$nB=1:nrow(c1)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  resR2p=data.frame(var=names(apply(rr3[,2:ncol(rr3)],2,mean)),moy=apply(rr3[,2:ncol(rr3)],2,mean),sd=apply(rr3[,2:ncol(rr3)],2,sd),median=apply(rr3[,2:ncol(rr3)],2,median),q025=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.025),q975=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.975))
  
}


write.table(resR2,"summary_R2.csv",sep=";",dec=".",row=F)
#R2 statistic computed from 10000 bootstrap model:
#type= R2 type (r2m= marginal R2; r2c= conditional R2)
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution
#n_sing= number of singular model


write.table(res,"summary_coeff.csv",sep=";",dec=".",row=F)
#coefficient statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap coefficient distribution
#q975 = 97.5% quantile of the bootstrap coefficient distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient lower than 0)


write.table(res2,"summary_randI.csv",sep=";",dec=".",row=F)
#tandom intercept statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap random intercept distribution
#q975 = 97.5% quantile of the bootstrap random intercept distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient lower than 0)

write.table(resR2p,"summary_R2p.csv",sep=";",dec=".",row=F)
#R2 partitioning computed from 10000 bootstrap model:
#var= Term of the model
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution

###4#Figure 1
library(pals) #version 1.6 

#Panel A
##Variance partitioning
setwd(outF)
t1=read.csv2("summary_R2p.csv",sep=";",dec=".",h=T)
t1=t1[order(t1$moy,decreasing=F),]
t1=t1[c(3,1:2,4:5),]
t1$n=1:5

png("Fig1.png",width=3.543*2,height=3.15,units="in",pointsize = 11,res=500)
def.par <- par(no.readonly = TRUE)
zones <- matrix(c(1,2,3), ncol = 3, byrow = TRUE)
nf=layout(zones, widths=c(5.25,5.25,1), heights = 5)

par(mai=c(0.36,1,0.05,0.05),ps=11,xpd=T)
plot(n~moy,ylim=c(0,5),xlim=c(0,50),data=t1,type="n",axes=F,xlab="", ylab="")
axis(1, seq(0,50,by=10),labels=F,tcl=-0.2,lwd=1) #y
axis(2,seq(0,5,by=1),labels=F,tcl=-0.2,lwd=1) #x
text(x=seq(0,50,by=10),y=-0.4,labels=as.character(seq(0,50,by=10)),xpd=NA,cex=0.9)
text(x=-2,y=seq(0.45,4.45,by=1),labels=rev(c("MacroTC",expression(paste(Delta,"Tbuff",sep="")),"Region",expression(paste("MacroTC x ",Delta,"Tbuff",sep="")),"Residuals")),xpd=NA,cex=0.9,srt=0,pos=2)
text(y=-0.7,x=25,labels="Proportion of variation explained (%)",cex=1.2,srt=0)
text(y=5.1,x=-22,labels="A",cex=1.1,font=2)

for(i in 1:5){
  polygon(y=c(i-1,i,i,i-1),x=c(t1$moy[i]*100,t1$moy[i]*100,0,0),lwd=1)#,col=1)
  #bar error for conditional r-square
  sdmax=t1$q975[i]
  sdmin=t1$q025[i]
  segments(y0=0.5+(i-1),y1=0.5+(i-1),x0=t1$moy[i]*100,x1=sdmax*100)
  segments(y0=0.45+(i-1),y1=0.55+(i-1),x0=sdmax*100,x1=sdmax*100)
  segments(y0=0.45+(i-1),y1=0.55+(i-1),x0=sdmin*100,x1=sdmin*100)#,col="white")
  segments(y0=0.5+(i-1),y1=0.5+(i-1),x0=sdmin*100,x1=t1$moy[i]*100)#,col="white")
}


#Panel B
setwd(inF)
t1=read.csv2("DataThermophilisation_allSpecies_Zellweger_et_al_Microclimate.csv",sep=",",dec=".",h=T) #all species
t1$microD=t1$MicroTmaxChangerate-t1$Thermophi_95pctl
t1$Tbuffd=(t1$TmaxBufResurvey-t1$TmaxBufBaseline) * -1 #Tbuff= micro - macro

setwd(outF)
r1=read.csv2("summary_coeff.csv",sep=";",dec=".",h=T)

#microclimatic debt predictions based on macroclimate change and changes in temperature buffering effect
x1 <- seq(-0.3, 1.1, by=0.1)
x2 <- seq(-3.4, 2.8, by=0.2)
newdata2<-expand.grid(x1=x1, x2=x2)

x1a <- (x1-mean(t1$MacroTmaxChangerate))/sd(t1$MacroTmaxChangerate)
x2a <- (x2-mean(t1$Tbuffd))/sd(t1$Tbuffd)
newdata<-expand.grid(x1a=x1a, x2a=x2a)

coef=r1$moy
newdata2$z=coef[1]+coef[2]*newdata$x1a+coef[4]*newdata$x2a+coef[3]*newdata$x2a*newdata$x1a

#color palette
warmcool2=rev(ocean.balance(length(seq(-3.4, 3.4, by=0.2)))[4:35])

par(mai=c(0.4,0.4,0.1,0),xpd=T,ps=11)
plot(z~x1,data=newdata2,ylim=c(-1,3),xlim=c(-0.4,1.2),type="n",axes=F,xlab="",ylab="",cex.axis=0.7)#xlab="Species richness", ylab="Mean forest productivity (m²/ha/5years)") #en ha?
axis(2, seq(-1,3,by=1),labels=F,tcl=-0.2,lwd=1)
axis(1,seq(-0.4,1.2,by=0.2),labels=F,tcl=-0.2,lwd=1)
text(x=-0.52,y=seq(-1,3,by=1),labels=as.character(seq(-1,3,by=1)),xpd=NA,cex=0.9)
text(x=seq(-0.4,1.2,by=0.2),y=-1.35,labels=as.character(seq(-0.4,1.2,by=0.2)),xpd=NA,cex=0.9)
text(x=0.4,y=-1.65,labels=expression(paste("Macroclimate Tmax change (°C.decade"^"-1",")",sep="")),cex=1.2)
text(y=1,x=-0.65,labels=expression(paste("Microclimatic debt (°C.decade"^"-1",")",sep="")),cex=1.2,srt=90)
text(y=3.2,x=-0.65,labels="B",cex=1.1,font=2)

points(microD~MacroTmaxChangerate,data=t1,pch=21,lwd=0.75,col="darkgrey",cex=0.5,bg=rgb(1,1,1,0.2))

newdata2=data.frame(newdata2)
v2=x2
for(i in 1:length(v2)){
  p1a=subset(newdata2,newdata2$x2==v2[i])
  points(z~x1,data=p1a,type="l",col=warmcool2[i+1],lwd=0.9)
}


#legend
par(mai=c(0.4,0,0.1,0),xpd=T,ps=11)
dim2=par()
l1=v2
n1=1:length(l1)
plot(x=c(-10,0),y=c(-3.4,2.8), axes = FALSE, xlim = c(-10,0),ylim=c(-3.4,2.8),xlab="", ylab="",
     space = 0, horiz = TRUE,type="n")
legend_image <- as.raster(matrix(rev(warmcool2), ncol=1))
text(x=-5.5,y=c(-3.4,-3,-2,-1,0,1,2,2.8),labels=as.character(c(-3.4,-3,-2,-1,0,1,2,2.8)),cex=0.9,xpd=NA) #as.character(round(c(l1[1],l1[125],l1[250],l1[375],l1[500],l1[625]),1))
rasterImage(legend_image, -10, -3.4, -8,2.8)
segments(x0=-10,x1=-10,y1=-3.4,y0=2.8)
segments(x0=-8,x1=-8,y1=-3.4,y0=2.8)
segments(x0=-10,x1=-8,y1=-3.4,y0=-3.4)
segments(x0=-10,x1=-8,y1=2.8,y0=2.8)
segments(x0=rep(-8,6),x1=rep(-7.5,6),y0=c(-3.4,-3,-2,-1,0,1,2,2.8),y1=c(-3.4,-3,-2,-1,0,1,2,2.8))
text(y=-0.3,x=-2,labels="Change in Tmax buffering (°C)",srt=270,cex=1.2)

dev.off()

###4#Testing for the effect of global warming for extreme change in temperature buffering due to canopy closure and openness
#mean effect of global warming when change in temperature buffering is the highest observed in the data (+2.8°C)
setwd(outF)
r1=read.csv2("summary_coeff.csv",sep=";",dec=".",h=T)
x2=2.8 #temperature buffering value for the highest canopy closure observed in data
x2a <- (x2-mean(t1$Tbuffd))/sd(t1$Tbuffd) #scaling
slop=r1$moy[2]+r1$moy[3]*x2a #global warming effect = 0.07257073

#Is this global warming effect on microclimatic debt significant? 
##H0: global warming effect = 0 
##tested through bootstrap.
setwd(outF)
Bselect=read.csv2("r2_10000boot.csv",sep=";",dec=".",h=T) #10000 selected bootstrap
coeff=read.csv2("coeff.csv",sep=";",dec=".",h=T) #coefficient estimates from the all set of bootstrapped models
nB=12000
coeff$nB=rep(1:nB,each=nrow(coeff)/nB)
slop=NULL
for(i in Bselect$nB){
  coeff2=subset(coeff,nB==i)
  slop=c(slop,coeff2$Estimate[coeff2$var=="scale(MacroTmaxChangerate)"]+coeff2$Estimate[coeff2$var=="scale(MacroTmaxChangerate):scale(Tbuffd)"]*x2a)
}
test1=function(x,mu=0){
  return(1-(length(x[x>mu])/length(x)))
}
test1(slop) #P<10^-4


#mean effect of global warming when change in temperature buffering is the lowest observed in the data (-3.4°C)
setwd(outF)
r1=read.csv2("summary_coeff.csv",sep=";",dec=".",h=T)
x2=-3.4 #change in temperature buffering value for the highest canopy openness observed in data
x2a <- (x2-mean(t1$Tbuffd))/sd(t1$Tbuffd) #scaling
slop=r1$moy[2]+r1$moy[3]*x2a #global warming effect = 0.4699825
slop/0.07257073 # global warming effect is 6.5 times higher than when high canopy closure occurred

#########################analysis of common species only########################
###data loading###
#data of Zellweger et al. 2020 are available here: https://doi.org/10.5061/dryad.r7sqv9s83
setwd(inF)
t1=read.csv2("DataThermophilisation_NorareSpecies_Zellweger_et_al_Microclimate.csv",sep=",",dec=".",h=T) #common species only

###varaible computation###
t1$microD=t1$MicroTmaxChangerate-t1$Thermophi_95pctl_NorareSp #the microclimatic debt as computed by Zellweger et al. (2020)
t1$Tbuffd=(t1$TmaxBufResurvey-t1$TmaxBufBaseline) * -1 #the change in temperature buffering by canopy closure and openness as computed by Zellweger et al. (2020)

###Thermophilization analysis###
#testing for the effect of changes in microclimate conditions on the thermophilization of forest plant communities (95th percentiles)
## model of Zellenberg et al. (2020)
M2<-lmer(Thermophi_95pctl_NorareSp ~ scale(MicroTmaxChangerate) + (1|region),data=t1,na.action=na.omit,REML=T)
r.squaredGLMM(M2) #R2m=0.02380336 & R2c=0.1356798

##Analsys of Bertrand et al. 2020
###output repository definition###
outF="..." #please set the correct reprository address where you want to save results (give a different address for each analysis)

#testing for the effect of macroclimate change and changes in temperature buffering by canopy closure and openness on the thermophilization observed in forest plant communities (95th percentiles)
###1#is collinearity between macroclimate change and temperature buffering change?
m1=lmer(scale(Tbuffd)~scale(MacroTmaxChangerate)+(1|region),data=t1,REML=T,na.action="na.fail",control = lmerControl(optimizer ='optimx', optCtrl=list(method='nlminb')))
r.squaredGLMM(m1) #R2m=0.006496959 & R2c=0.2466336
###No collinearity

###1#LMM
mX=lmer(Thermophi_95pctl_NorareSp~scale(MacroTmaxChangerate)*scale(Tbuffd)+(1|region),data=t1,REML=T,na.action="na.fail",control = lmerControl(optimizer ='optimx', optCtrl=list(method='nlminb')))
summary(mX)
r.squaredGLMM(mX) #R2m=0.01373782 & R2c=0.1280631

###2#Computing bootstraps to infer 95% confidence intervals of R2 (R2c, R2M and part of variance explained by each direct effects using the function calcVarPart from the variancePartition package) and size effects, and also to test for significance of each direct effects.
r2=NULL
nB=12000 #set the bootstrap number
s1=5 #the seed
ncpu=10 #the number of cpu used for parallel computation

#WARNING: it can take a long time to run. It takes ~13min on 10 CPUs of a Intel® Xeon(R) E-2286M CPU @ 2.40GHz × 16 computer with 32Go memory (ubuntu 18.04)
t0=Sys.time()
boot1.time <- system.time(
  boot1 <- bootMer2(mX, statF, nsim=nB, use.u=F, type="parametric",ncpus=ncpu,seed=s1,parallel="multicore",.progress = "none")
)
print(Sys.time()-t0)

#exporting results
ess=simplify2array(boot1)
r2=rbindlist(ess[1,])
coeff=rbindlist(ess[2,])
rand=rbindlist(ess[3,])
rand2=rbindlist(ess[4,])
vPart=rbindlist(ess[5,])

setwd(outF)  
write.table(r2,"r2.csv",sep=";",dec=".",row=F)
#output:
#r2m= marginal R2 of every bootstrap model (each line is a different model)
#r2c= conditional R2 of every bootstrap model
#warnM= inform for warning during the model fit (such as convergence issue)
#sing= output of the model singularity test (FALSE= no signularity; TRUE= singularity issue)

write.table(coeff,"coeff.csv",sep=";",dec=".",row=F)
#output is the result of model$coefficient for everey bootstrap model:
#Estimate= coefficient value computed from every bootstrap model
#Std..Error= Standard error computed from every bootstrap model
#t.value= t-value computed from every bootstrap model
#var= names of the term

write.table(rand2,"randVar.csv",sep=";",dec=".",row=F)
#output is the result of summary(.)$varcor for every bootstrap model:
#var= names of the term
#vcov= Variance of random intercept

write.table(vPart,"vPart.csv",sep=";",dec=".",row=F)
#output is the result of variance partitioning inferred from calcVarPart function (variancePartition package)
#each columns is a variable
#each row is a bootstrap iteration

###3# Bootstrap analysis
setwd(outF) 
vPart=read.csv2("vPart.csv",sep=";",dec=".",h=T)
coeff=read.csv2("coeff.csv",sep=";",dec=".",h=T)
rand2=read.csv2("randVar.csv",sep=";",dec=".",h=T)
r2=read.csv2("r2.csv",sep=";",dec=".",h=T)

#bootstrap test: H0= coefficient equal to 0; H1= coefficient higher than 0
test1=function(x,mu=0){
  return(1-(length(x[x>mu])/length(x)))
}

#bootstrap test: H0= coefficient equal to 0; H1= coefficient lower than 0
test2=function(x,mu=0){
  return(1-(length(x[x<mu])/length(x)))
}

nB2=10000 #the number of bootstrap selection.
#Some of the 12000 bootstraps previously conducted can be biased by singularity or model convergence issues.
#So we selected the 10000 bootstraps withouth any fitting issues
s1=10 #the random seed
r2$nB=1:nrow(r2)
r2a=subset(r2,is.na(warnM)==T)
if(nrow(r2a)>=nB2){
  set.seed(s1)
  r2a=r2a[sample(1:nrow(r2a),size=nB2,replace=F),]
}
if(nrow(r2a)<nB2){
  print(paste("sampling size is less than ",nB2," bootstraps",sep=""))
}else{
  write.table(r2a,paste("r2_",nB2,"boot.csv",sep=""),sep=";",dec=".",row=F) #the list of plots selected for each boostrap iteration
  resR2=data.frame(type="r2m",moy=mean(r2a$r2m),sd=sd(r2a$r2m),median=median(r2a$r2m),q025=quantile(r2a$r2m,probs=0.025),q975=quantile(r2a$r2m,probs=0.975))
  resR2=rbind(resR2,data.frame(type="r2c",moy=mean(r2a$r2c),sd=sd(r2a$r2c),median=median(r2a$r2c),q025=quantile(r2a$r2c,probs=0.025),q975=quantile(r2a$r2c,probs=0.975)))
  resR2$n_sing=nrow(subset(r2a,sing==T))
  
  c1=coeff
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res=data.frame(var=names(tapply(rr3$Estimate,rr3$var,mean)),moy=tapply(rr3$Estimate,rr3$var,mean),sd=tapply(rr3$Estimate,rr3$var,sd),median=tapply(rr3$Estimate,rr3$var,median),q025=tapply(rr3$Estimate,rr3$var,quantile,probs=0.025),
                 p975=tapply(rr3$Estimate,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$Estimate,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$Estimate,rr3$var,test1,mu=0))
  
  c1=rand2
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res2=data.frame(var=names(tapply(rr3$vcov,rr3$var,mean)),moy=tapply(rr3$vcov,rr3$var,mean),sd=tapply(rr3$vcov,rr3$var,sd),median=tapply(rr3$vcov,rr3$var,median),q025=tapply(rr3$vcov,rr3$var,quantile,probs=0.025),
                  p975=tapply(rr3$vcov,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$vcov,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$vcov,rr3$var,test1,mu=0))
  
  c1=vPart
  c1$nB=1:nrow(c1)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  resR2p=data.frame(var=names(apply(rr3[,2:ncol(rr3)],2,mean)),moy=apply(rr3[,2:ncol(rr3)],2,mean),sd=apply(rr3[,2:ncol(rr3)],2,sd),median=apply(rr3[,2:ncol(rr3)],2,median),q025=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.025),q975=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.975))
  
}

write.table(resR2,"summary_R2.csv",sep=";",dec=".",row=F)
#R2 statistic computed from 10000 bootstrap model:
#type= R2 type (r2m= marginal R2; r2c= conditional R2)
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution
#n_sing= number of singular model


write.table(res,"summary_coeff.csv",sep=";",dec=".",row=F)
#coefficient statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap coefficient distribution
#q975 = 97.5% quantile of the bootstrap coefficient distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient lower than 0)


write.table(res2,"summary_randI.csv",sep=";",dec=".",row=F)
#random intercept statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap random intercept distribution
#q975 = 97.5% quantile of the bootstrap random intercept distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient lower than 0)

write.table(resR2p,"summary_R2p.csv",sep=";",dec=".",row=F)
#R2 partitioning computed from 10000 boostrap model:
#var= Term of the model
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution

###Microclimatic debt analysis###
#Analsys of Bertrand et al. 2020
###output repository definition###
outF="..." #please set the correct reprository address where you want to save results (give a different address for each analysis)

#testing for the effect of macroclimate change and changes in temperature buffering by canopy closure and openness on the microclimatic debt observed in forest plant communities (95th percentiles)
###1#LMM
mX=lmer(microD~scale(MacroTmaxChangerate)*scale(Tbuffd)+(1|region),data=t1,REML=T,na.action="na.fail",control = lmerControl(optimizer ='optimx', optCtrl=list(method='nlminb')))
summary(mX)
r.squaredGLMM(mX) #R2m=0.6350381 & R2c=0.6695924

###2#Computing bootstraps to infer 95% confidence intervals of R2 (R2c, R2M and part of variance explained by each direct effects using the function calcVarPart from the variancePartition package) and size effects, and also to test for significance of each direct effects.
r2=NULL
nB=12000 #set the bootstrap number
s1=5 #the seed
ncpu=10 #the number of cpu used for parallel computation

#WARNING: it can take a long time to run. It takes ~13min on 10 CPUs of a Intel® Xeon(R) E-2286M CPU @ 2.40GHz × 16 computer with 32Go memory (ubuntu 18.04)
t0=Sys.time()
boot1.time <- system.time(
  boot1 <- bootMer2(mX, statF, nsim=nB, use.u=F, type="parametric",ncpus=ncpu,seed=s1,parallel="multicore",.progress = "none")
)
print(Sys.time()-t0)

#exporting results
ess=simplify2array(boot1)
r2=rbindlist(ess[1,])
coeff=rbindlist(ess[2,])
rand=rbindlist(ess[3,])
rand2=rbindlist(ess[4,])
vPart=rbindlist(ess[5,])

setwd(outF)  
write.table(r2,"r2.csv",sep=";",dec=".",row=F)
#output:
#r2m= marginal R2 of every bootstrap model (each line is a different model)
#r2c= conditional R2 of every bootstrap model
#warnM= inform for warning during the model fit (such as convergence issue)
#sing= output of the model singularity test (FALSE= no signularity; TRUE= singularity issue)

write.table(coeff,"coeff.csv",sep=";",dec=".",row=F)
#output is the result of model$coefficient for everey bootstrap model:
#Estimate= coefficient value computed from every bootstrap model
#Std..Error= Standard error computed from every bootstrap model
#t.value= t-value computed from every bootstrap model
#var= names of the term

write.table(rand2,"randVar.csv",sep=";",dec=".",row=F)
#output is the result of summary(.)$varcor for everey bootstrap model:
#var= names of the term
#vcov= Variance of random intercept

write.table(vPart,"vPart.csv",sep=";",dec=".",row=F)
#output is the result of variance partitioning inferred from calcVarPart function (variancePartition package)
#each columns is a variable
#each row is a bootstrap iteration

###3# Bootstrap analysis
setwd(outF) 
vPart=read.csv2("vPart.csv",sep=";",dec=".",h=T)
coeff=read.csv2("coeff.csv",sep=";",dec=".",h=T)
rand2=read.csv2("randVar.csv",sep=";",dec=".",h=T)
r2=read.csv2("r2.csv",sep=";",dec=".",h=T)

#bootstrap test: H0= coefficient equal to 0; H1= coefficient higher than 0
test1=function(x,mu=0){
  return(1-(length(x[x>mu])/length(x)))
}

#bootstrap test: H0= coefficient equal to 0; H1= coefficient lower than 0
test2=function(x,mu=0){
  return(1-(length(x[x<mu])/length(x)))
}

nB2=10000 #the number of bootstrap selection.
#Some of the 12000 bootstraps previously conducted can be biased by singularity or model convergence issues.
#So we selected the 10000 bootstraps withouth any fitting issues
s1=10 #the random seed
r2$nB=1:nrow(r2)
r2a=subset(r2,is.na(warnM)==T)
if(nrow(r2a)>=nB2){
  set.seed(s1)
  r2a=r2a[sample(1:nrow(r2a),size=nB2,replace=F),]
}
if(nrow(r2a)<nB2){
  print(paste("sampling size is less than ",nB2," bootstraps",sep=""))
}else{
  write.table(r2a,paste("r2_",nB2,"boot.csv",sep=""),sep=";",dec=".",row=F) #the list of plots selected for each boostrap iteration
  resR2=data.frame(type="r2m",moy=mean(r2a$r2m),sd=sd(r2a$r2m),median=median(r2a$r2m),q025=quantile(r2a$r2m,probs=0.025),q975=quantile(r2a$r2m,probs=0.975))
  resR2=rbind(resR2,data.frame(type="r2c",moy=mean(r2a$r2c),sd=sd(r2a$r2c),median=median(r2a$r2c),q025=quantile(r2a$r2c,probs=0.025),q975=quantile(r2a$r2c,probs=0.975)))
  resR2$n_sing=nrow(subset(r2a,sing==T))
  
  c1=coeff
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res=data.frame(var=names(tapply(rr3$Estimate,rr3$var,mean)),moy=tapply(rr3$Estimate,rr3$var,mean),sd=tapply(rr3$Estimate,rr3$var,sd),median=tapply(rr3$Estimate,rr3$var,median),q025=tapply(rr3$Estimate,rr3$var,quantile,probs=0.025),
                 p975=tapply(rr3$Estimate,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$Estimate,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$Estimate,rr3$var,test1,mu=0))
  
  c1=rand2
  c1$nB=rep(1:nB,each=nrow(c1)/nB)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  
  res2=data.frame(var=names(tapply(rr3$vcov,rr3$var,mean)),moy=tapply(rr3$vcov,rr3$var,mean),sd=tapply(rr3$vcov,rr3$var,sd),median=tapply(rr3$vcov,rr3$var,median),q025=tapply(rr3$vcov,rr3$var,quantile,probs=0.025),
                  p975=tapply(rr3$vcov,rr3$var,quantile,probs=0.975), pv.inf0=tapply(rr3$vcov,rr3$var,test2,mu=0),pv.sup0=tapply(rr3$vcov,rr3$var,test1,mu=0))
  
  c1=vPart
  c1$nB=1:nrow(c1)
  c1=merge(c1,data.frame(nB=r2a$nB),by.x="nB",by.y="nB")
  rr3=c1
  resR2p=data.frame(var=names(apply(rr3[,2:ncol(rr3)],2,mean)),moy=apply(rr3[,2:ncol(rr3)],2,mean),sd=apply(rr3[,2:ncol(rr3)],2,sd),median=apply(rr3[,2:ncol(rr3)],2,median),q025=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.025),q975=apply(rr3[,2:ncol(rr3)],2,quantile,probs=0.975))
  
}


write.table(resR2,"summary_R2.csv",sep=";",dec=".",row=F)
#R2 statistic computed from 10000 bootstrap model:
#type= R2 type (r2m= marginal R2; r2c= conditional R2)
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution
#n_sing= number of singular model


write.table(res,"summary_coeff.csv",sep=";",dec=".",row=F)
#coefficient statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap coefficient distribution
#q975 = 97.5% quantile of the bootstrap coefficient distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= coefficient equal to 0; H1= coefficient lower than 0)


write.table(res2,"summary_randI.csv",sep=";",dec=".",row=F)
#random intercept statistics
#Var =  Term of the model
#moy = mean effect computed from the 10000 bootstrap model
#sd = standard deviation computed from the 10000 bootstrap model
#med = median effect computed from the 10000 bootstrap model
#q025 = 2.5% quantile of the bootstrap random intercept distribution
#q975 = 97.5% quantile of the bootstrap random intercept distribution
#pv.sup0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient greater than 0)
#pv.inf0 = p-value of the bootstrap test testing the significance of the effect (H0= random intercept variation equal to 0; H1= coefficient lower than 0)

write.table(resR2p,"summary_R2p.csv",sep=";",dec=".",row=F)
#R2 partitioning computed from 10000 bootstrap model:
#var= Term of the model
#moy= mean R2 value
#sd= R2 standard deviation value
#median= median R2 value
#q025= 2.5% quantile of the bootstrap distribution
#q975= 97.5% quantile of the bootstrap distribution

###4#Figure 1 for common species only
library(pals) #version 1.6 

#Panel A
##Variance partitioning
setwd(outF)
t1=read.csv2("summary_R2p.csv",sep=";",dec=".",h=T)
t1=t1[order(t1$moy,decreasing=F),]
t1=t1[c(3,1:2,4:5),]
t1$n=1:5

png("Fig1.png",width=3.543*2,height=3.15,units="in",pointsize = 11,res=500)
def.par <- par(no.readonly = TRUE)
zones <- matrix(c(1,2,3), ncol = 3, byrow = TRUE)
nf=layout(zones, widths=c(5.25,5.25,1), heights = 5)

par(mai=c(0.36,1,0.05,0.05),ps=11,xpd=T)
plot(n~moy,ylim=c(0,5),xlim=c(0,50),data=t1,type="n",axes=F,xlab="", ylab="")
axis(1, seq(0,50,by=10),labels=F,tcl=-0.2,lwd=1) #y
axis(2,seq(0,5,by=1),labels=F,tcl=-0.2,lwd=1) #x
text(x=seq(0,50,by=10),y=-0.4,labels=as.character(seq(0,50,by=10)),xpd=NA,cex=0.9)
text(x=-2,y=seq(0.45,4.45,by=1),labels=rev(c("MacroTC",expression(paste(Delta,"Tbuff",sep="")),"Region",expression(paste("MacroTC x ",Delta,"Tbuff",sep="")),"Residuals")),xpd=NA,cex=0.9,srt=0,pos=2)
text(y=-0.7,x=25,labels="Proportion of variation explained (%)",cex=1.2,srt=0)
text(y=5.1,x=-22,labels="A",cex=1.1,font=2)

for(i in 1:5){
  polygon(y=c(i-1,i,i,i-1),x=c(t1$moy[i]*100,t1$moy[i]*100,0,0),lwd=1)#,col=1)
  #bar error for conditional r-square
  sdmax=t1$q975[i]
  sdmin=t1$q025[i]
  segments(y0=0.5+(i-1),y1=0.5+(i-1),x0=t1$moy[i]*100,x1=sdmax*100)
  segments(y0=0.45+(i-1),y1=0.55+(i-1),x0=sdmax*100,x1=sdmax*100)
  segments(y0=0.45+(i-1),y1=0.55+(i-1),x0=sdmin*100,x1=sdmin*100)#,col="white")
  segments(y0=0.5+(i-1),y1=0.5+(i-1),x0=sdmin*100,x1=t1$moy[i]*100)#,col="white")
}


#Panel B
setwd(inF)
t1=read.csv2("DataThermophilisation_allSpecies_Zellweger_et_al_Microclimate.csv",sep=",",dec=".",h=T) #all species
t1$microD=t1$MicroTmaxChangerate-t1$Thermophi_95pctl
t1$Tbuffd=(t1$TmaxBufResurvey-t1$TmaxBufBaseline) * -1 #Tbuff= micro - macro

setwd(outF)
r1=read.csv2("summary_coeff.csv",sep=";",dec=".",h=T)

#microclimatic debt predictions based on macroclimate change and temperature buffering effect
x1 <- seq(-0.3, 1.1, by=0.1)
x2 <- seq(-3.4, 2.8, by=0.2)
newdata2<-expand.grid(x1=x1, x2=x2)

x1a <- (x1-mean(t1$MacroTmaxChangerate))/sd(t1$MacroTmaxChangerate)
x2a <- (x2-mean(t1$Tbuffd))/sd(t1$Tbuffd)
newdata<-expand.grid(x1a=x1a, x2a=x2a)

coef=r1$moy
newdata2$z=coef[1]+coef[2]*newdata$x1a+coef[4]*newdata$x2a+coef[3]*newdata$x2a*newdata$x1a

#color palette
warmcool2=rev(ocean.balance(length(seq(-3.4, 3.4, by=0.2)))[4:35])

par(mai=c(0.4,0.4,0.1,0),xpd=T,ps=11)
plot(z~x1,data=newdata2,ylim=c(-1,3),xlim=c(-0.4,1.2),type="n",axes=F,xlab="",ylab="",cex.axis=0.7)#xlab="Species richness", ylab="Mean forest productivity (m²/ha/5years)") #en ha?
axis(2, seq(-1,3,by=1),labels=F,tcl=-0.2,lwd=1)
axis(1,seq(-0.4,1.2,by=0.2),labels=F,tcl=-0.2,lwd=1)
text(x=-0.52,y=seq(-1,3,by=1),labels=as.character(seq(-1,3,by=1)),xpd=NA,cex=0.9)
text(x=seq(-0.4,1.2,by=0.2),y=-1.35,labels=as.character(seq(-0.4,1.2,by=0.2)),xpd=NA,cex=0.9)
text(x=0.4,y=-1.65,labels=expression(paste("Macroclimate Tmax change (°C.decade"^"-1",")",sep="")),cex=1.2)
text(y=1,x=-0.65,labels=expression(paste("Microclimatic debt (°C.decade"^"-1",")",sep="")),cex=1.2,srt=90)
text(y=3.2,x=-0.65,labels="B",cex=1.1,font=2)

points(microD~MacroTmaxChangerate,data=t1,pch=21,lwd=0.75,col="darkgrey",cex=0.5,bg=rgb(1,1,1,0.2))

newdata2=data.frame(newdata2)
v2=x2
for(i in 1:length(v2)){
  p1a=subset(newdata2,newdata2$x2==v2[i])
  points(z~x1,data=p1a,type="l",col=warmcool2[i+1],lwd=0.9)
}


#legend
par(mai=c(0.4,0,0.1,0),xpd=T,ps=11)
dim2=par()
l1=v2
n1=1:length(l1)
plot(x=c(-10,0),y=c(-3.4,2.8), axes = FALSE, xlim = c(-10,0),ylim=c(-3.4,2.8),xlab="", ylab="",
     space = 0, horiz = TRUE,type="n")
legend_image <- as.raster(matrix(rev(warmcool2), ncol=1))
text(x=-5.5,y=c(-3.4,-3,-2,-1,0,1,2,2.8),labels=as.character(c(-3.4,-3,-2,-1,0,1,2,2.8)),cex=0.9,xpd=NA) #as.character(round(c(l1[1],l1[125],l1[250],l1[375],l1[500],l1[625]),1))
rasterImage(legend_image, -10, -3.4, -8,2.8)
segments(x0=-10,x1=-10,y1=-3.4,y0=2.8)
segments(x0=-8,x1=-8,y1=-3.4,y0=2.8)
segments(x0=-10,x1=-8,y1=-3.4,y0=-3.4)
segments(x0=-10,x1=-8,y1=2.8,y0=2.8)
segments(x0=rep(-8,6),x1=rep(-7.5,6),y0=c(-3.4,-3,-2,-1,0,1,2,2.8),y1=c(-3.4,-3,-2,-1,0,1,2,2.8))
text(y=-0.3,x=-2,labels="Change in Tmax buffering (°C)",srt=270,cex=1.2)

dev.off()

###4#Testing for the effect of global warming for extreme change in temperature buffering due to canopy closure and openness
#mean effect of global warming when change in temperature buffering is the highest observed in the data (+2.8°C)
setwd(outF)
r1=read.csv2("summary_coeff.csv",sep=";",dec=".",h=T)
x2=2.8 #temperature buffering value for the highest canopy closure observed in data
x2a <- (x2-mean(t1$Tbuffd))/sd(t1$Tbuffd) #scaling
slop=r1$moy[2]+r1$moy[3]*x2a #global warming effect = 0.05554864

#Is this global warming effect on microclimatic debt significant? 
##H0: global warming effect = 0 
##tested through bootstrap.
setwd(outF)
Bselect=read.csv2("r2_10000boot.csv",sep=";",dec=".",h=T) #10000 selected bootstrap
coeff=read.csv2("coeff.csv",sep=";",dec=".",h=T) #coefficient estimates from the all set of bootstrapped models
nB=12000
coeff$nB=rep(1:nB,each=nrow(coeff)/nB)
slop=NULL
for(i in Bselect$nB){
  coeff2=subset(coeff,nB==i)
  slop=c(slop,coeff2$Estimate[coeff2$var=="scale(MacroTmaxChangerate)"]+coeff2$Estimate[coeff2$var=="scale(MacroTmaxChangerate):scale(Tbuffd)"]*x2a)
}
test1=function(x,mu=0){
  return(1-(length(x[x>mu])/length(x)))
}
test1(slop) #P=0.0023


#mean effect of global warming when change in temperature buffering is the lowest observed in the data (-3.4°C)
setwd(outF)
r1=read.csv2("summary_coeff.csv",sep=";",dec=".",h=T)
x2=-3.4 #change in temperature buffering value for the highest canopy openness observed in data
x2a <- (x2-mean(t1$Tbuffd))/sd(t1$Tbuffd) #scaling
slop=r1$moy[2]+r1$moy[3]*x2a #global warming effect = 0.4699825
slop/0.05554864 # global warming effect is 8.3 times higher than when high canopy closure occurred
###END